# To be called with full path of this script!

use strict "vars";

# Perl script to collect SCC-data.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: http://scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.20 $


my $PROGNAME = "$0";

my $computer = $ENV{ SCC_WMI_COMPUTER };
if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME for $computer \n";
}

my $SYNTAX = "Syntax error, use: $PROGNAME [ --extra <mod> ]\n";

my $extra_mod = "";
while ( @ARGV )
{
	if ( "$ARGV[ 0 ]" =~ /^--extra$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$extra_mod = "$ARGV[ 0 ]";
		}
		shift( @ARGV );
	}
	else
	{
		die "$SYNTAX";
	}
}

# Determine our directories.
my $dir = $0;
$dir =~ s/[^\\]*$//;		# Erase the name of the script.
$dir =~ s/[^\\]*\\$//;		# Erase deepest directory (bin)

my $SCC_DATA	= $dir		. "data\\";
my $SCC_TMP	= $dir		. "tmp\\";
my $SCC_BIN	= $dir		. "bin\\";
my $SCC_MODULES	= $SCC_BIN	. "scc_modules\\";

mkdir( $SCC_DATA . "plugin_data", 0777 );

$ENV{ SCC_BIN }	= $SCC_BIN;
$ENV{ SCC_TMP }	= $SCC_TMP;
$ENV{ SCC_DATA } = $SCC_DATA;

chdir( $SCC_TMP ) or die "$PROGNAME: Cannot change directory to SCC_TMP";

use Win32::OLE('in');

use constant wbemFlagReturnImmediately => 0x10;
use constant wbemFlagForwardOnly => 0x20;

my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2") or die "$PROGNAME: WMI connection failed for $computer \n";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "  start Win32_OperatingSystem to determine OS-version\n";
}
my $OS_colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_OperatingSystem", "WQL",
			wbemFlagReturnImmediately | wbemFlagForwardOnly);
my $os_version = "";
foreach my $objItem (in $OS_colItems)
{
        $os_version = $objItem->{Version};
}
$os_version =~ s/\.[0-9][0-9]*$//;
# "6.0" System = "Vista"
# "5.2" System = "Windows 2003"
# "5.1" System = "XP"
# "5.0" System = "W2K"
# "4.0" System = "NT 4.0**"
# Else System = "Unknown
$ENV{ SCC_OS_VERSION } = $os_version;

my $tick_prev = time();
my $total_time = 0;

sub scc_timing ($)
{
	my $sec = 0;
	my $min = 0;
	my $hour = 0;
	my $dummy;
	my $label = shift;
	my $now = time();

	($sec,$min,$hour,$dummy,$dummy,$dummy,$dummy,$dummy,$dummy) = localtime( $now );
	$total_time += $now - $tick_prev;

	printf( "var:profiling::%02d.%02d.%02d: %4d : %4d :%s\n",
		$hour, $min, $sec, $now - $tick_prev, $total_time, $label );

	$tick_prev = $now;
}

scc_timing( "start of run" );

opendir( MODULES, $SCC_MODULES ) or die "$PROGNAME: Cannot access modules directory $SCC_MODULES ";
my @all_files = readdir MODULES;
closedir MODULES;

foreach my $module ( @all_files )
{
	if ( $module =~ "^scc_[0-9][0-9][0-9][0-9]_s_.*\.pl\$" )	# System module?
	{
		scc_timing( "$module	start of module" );

		my $mod = "$SCC_MODULES" . "$module";
		if ( system( ( "perl", "\"$mod\"" ) ) )
		{
			# Module died, for example: WMI connection failed.
			print STDERR "$PROGNAME: module $module failed: $! \n";
			exit 1;		# Signal failure
		}

		scc_timing( "$module	end of module" );
	}

	if ( $module =~ "^scc_[0-9][0-9][0-9][0-9]_u_.*\.pl\$" )	# User module?
	{
		# The output will be/has been recorded in in $SCC_DATA\plugin_data\scc.<host>.<mod>
		my $mod_data = $module;
		$mod_data =~ s/\.pl$//;
		$mod_data =~ s/.*scc_/scc_/;
		$mod_data = $SCC_DATA . "plugin_data\\scc." . $ENV{ SCC_WMI_COMPUTER } .  "." . $mod_data;
		$ENV{ SCC_PLUGIN_DATA } = $mod_data;	# To be used by user module.

		if ( ( $extra_mod ne "" ) && ( $module =~ $extra_mod ) )
		{
			scc_timing( "$module	start of module" );

			my $mod = "$SCC_MODULES" . "$module";
			if ( system( ( "perl", "\"$mod\"" ) ) )
			{
				# Module died, for example: WMI connection failed.
				print STDERR "$PROGNAME: module $module failed: $! \n";
				exit 1;		# Signal failure
			}

			open INPUT,"<$mod_data";
			while ( <INPUT> )
			{
				print $_;
			}
			close INPUT;

			scc_timing( "$module	end of module" );
		}
		if ( ( $extra_mod eq "" ) && ( -f $mod_data ) )
		{
			open INPUT,"<$mod_data";
			while ( <INPUT> )
			{
				print $_;
			}
			close INPUT;
		}
	}
}

scc_timing( "end of run" );

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

exit 0; 
