
# Perl script to convert a SCC logbook to html-format.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: http://scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.25 $


my $PROGNAME = "$0";

use strict "vars";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME \n";
}

my $host="$ARGV[0]";

my $tmp_1_file = "logp2html_1";
my $tmp_2_file = "logp2html_2";
my $tmp_3_file = "logp2html_3";

my $TABLE_TAG="<TABLE CLASS=SCC SUMMARY=\"Summary of scc-runs\" BORDER CELLSPACING=1 CELLPADDING=4>";
my $TOP_NAME="<A NAME=cfg_top></A>";
my $TOP_URL="<A HREF=\"#cfg_top\">Top</A>&nbsp;&nbsp;&nbsp;&nbsp;";

print "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
print "<HTML>\n";

print "<HEAD>\n";
print "<LINK HREF=\"style.css\" REL=\"stylesheet\" TYPE=\"text/css\">\n";
print "<TITLE>Logbook: $host</TITLE>\n";
print "</HEAD>\n";

print "<BODY>\n";
print "<DIV class=SCC_LOG>\n";
print "$TOP_NAME\n";

print "<H1>Logbook: $host</H1>\n";

print "<DIV class=SCC_LOG_NAV>\n";

# DO NOT CHANGE THE FOLLOWING HTML-CODE WITHOUT CONSULTING scc.cgi IN scc-srv
print "<H4>\n";
print "	<A HREF=\"index.html\">Home</A>&nbsp;&nbsp;&nbsp;&nbsp;\n";
print "	<A HREF=\"scc.$host.html\">Configuration</A>&nbsp;&nbsp;&nbsp;&nbsp;\n";
print "	<A HREF=\"#cfg_statistics\">Statistics</A>\n";
print "</H4>\n";
print "</DIV><!-- class=SCC_LOG_NAV -->\n";

print "<H4>Summary of runs of SCC</H4>\n";

open TMP_1,">$tmp_1_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	# Replace special HTML-characters in input.
	s/&/\&amp;/g;
	s/</\&lt;/g;
	s/>/\&gt;/g;
	s/'/\&#39;/g;
	s/"/\&quot;/g;

	print TMP_1 $_;
}
close STDIN;
close TMP_1;

# Build a table with URL's to reported differences in the logbook.
# Use the date/time of the run as the ID for HREF and NAME.
# Sort the data reverse to show the most recent change at the top of the table.
# Check the code in scc-log for the specific layout of the logbook.
my $stat_cnt_runs = 0;
my $prev_res = "";
my $prev_remark = "";
my $prev_runtime = "";
my $stat_runtime_cnt = 0;
my $stat_runtime_total = 0;
my $stat_runtime_max = 0;
my $stat_runtime_min = 0;
my $stat_change_cnt = 0;
my $stat_change_total = 0;
my $stat_change_max = 0;
my $stat_change_min = 0;
open STDIN,"<$tmp_1_file" || die "Could not open temporary-file";
open TMP_2,">$tmp_2_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	chomp;
	if ( $_ =~ ":result::" )
	{
		$stat_cnt_runs++;
		if ( $prev_res )
		{
			# restart or identical, no number of differences.
			print TMP_2 $prev_res . ":" . ":" . $prev_runtime . ":" . $prev_remark . "\n";
			$prev_remark = "";
			$prev_runtime = "";
		}
		$prev_res = $_;
		next;
	}
	if ( $_ =~ ":remark::" )
	{
		# A remark can contain ":", syntax of a line is: <date>:<time>:remark::<remark>
		s/::/ScC_MaRkEr::/;
		s/.*ScC_MaRkEr:://;
		$prev_remark = $_;
		next;
	}
	if ( $_ =~ ":runtime::" )
	{
		s/.*://;

		$stat_runtime_cnt++;
		$stat_runtime_total += $_;
		if ( $_ > $stat_runtime_max )
		{
			$stat_runtime_max = $_;
		}
		if ( $_ < $stat_runtime_min || $stat_runtime_min eq 0 )
		{
			$stat_runtime_min = $_;
		}

		$prev_runtime = $_;
		next;
	}
	if ( $_ =~ ":count::" )
	{
		s/.*://;

		$stat_change_cnt++;
		$stat_change_total += $_;
		if ( $_ > $stat_change_max )
		{
			$stat_change_max = $_;
		}
		if ( $_ < $stat_change_min || $stat_change_min eq 0 )
		{
			$stat_change_min = $_;
		}
		# changes detected, report
		print TMP_2 $prev_res . ":" . $_ . ":" . $prev_runtime . ":" . $prev_remark . "\n";
		$prev_res = "";
		$prev_remark = "";
		next;
	}
}
close STDIN;

# unreported results?
if ( $prev_res )
{
	print TMP_2 $prev_res . ":" . ":" . $prev_runtime . ":" . $prev_remark . "\n";
}
if ( $stat_cnt_runs eq 0 )
{
	$stat_cnt_runs = 1;
	$stat_runtime_cnt = 1;
}

my $tmp_cnt = 0;
print TMP_2 "0:9:runs total          : $stat_cnt_runs\n";
print TMP_2 "0:8:runs with changes   : $stat_change_cnt\n";
$tmp_cnt = sprintf "%d", ( 100 * $stat_change_cnt ) / $stat_cnt_runs;
print TMP_2 "0:7:runs perc. changes  : $tmp_cnt\n";
print TMP_2 "0:6:change count min.   : $stat_change_min\n";
print TMP_2 "0:5:change count max.   : $stat_change_max\n";
if ( $stat_change_cnt eq 0 )
{
	$stat_change_cnt = 1;
}
$tmp_cnt = sprintf "%d", $stat_change_total / $stat_change_cnt;
print TMP_2 "0:4:change count average: $tmp_cnt\n";
print TMP_2 "0:3:runtime min.        : $stat_runtime_min\n";
print TMP_2 "0:2:runtime max.        : $stat_runtime_max\n";
$tmp_cnt = sprintf "%d", $stat_runtime_total / $stat_runtime_cnt;
print TMP_2 "0:1:runtime average     : $tmp_cnt\n";
close TMP_2;

my $sort_exe = "$ENV{ SCC_BIN }" . "sort";
my @args =	(
			"\"$sort_exe\"",
			"-r",
			"-o", "$tmp_3_file",
			"$tmp_2_file"
		);
if ( system( @args ) )
{
	print STDERR "$PROGNAME: unable to sort statistics: $! \n";
	exit 1;				# Indicate failure.
}

print "<DIV class=SCC_LOG_SUMMARY>\n";

my @c = ( "Even", "Odd" );
my $c_i = 1;

print "$TABLE_TAG\n";
print "<THEAD>\n";
print "	<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
print "		<TH>Date</TH>\n";
print "		<TH>Time</TH>\n";
print "		<TH>Runtime</TH>\n";
print "		<TH>Result</TH>\n";
print "		<TH>Count</TH>\n";
print "		<TH>Remark</TH>\n";
print "	</TR>\n";
print "</THEAD>\n";
print "<TBODY>\n";

$c_i = 0;

my $statistics = 0;
my @field_l;
open STDIN,"<$tmp_3_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	chomp;
	if ( $_ =~ "^0:" )
	{
		# We print the statistics as pre-formatted text.
		# Last part of the data. The END-statement takes care of the </PRE>.
		# Format of the data is:
		#0:<order>:<label>:<value>
		if ( $statistics eq 0 )		# No heading yet?
		{
			print "</TBODY>\n";
			print "</TABLE>\n";
			print "</DIV><!-- class=SCC_LOG_SUMMARY -->\n";
			print "<HR>\n";
			print "<DIV class=SCC_LOG_STATS>\n";
			print "<A NAME=cfg_statistics></A>\n";
			print "<H4>" . $TOP_URL . "Statistics</H4>\n";
			
			print "$TABLE_TAG\n";
			print "<THEAD>\n";
			print "	<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
			print "		<TH>Category</TH>\n";
			print "		<TH>Value</TH>\n";
			print "	</TR>\n";
			print "</THEAD>\n";

			print "<TBODY>\n";
			$statistics = 1;
		}
		s/[0-9]*:[0-9]*://;
		my $cat = $_;
		$cat =~ s/:.*//;
		my $val = $_;
		$val =~ s/^[^:]*://;
		print "<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
		print "	<TD class=Even>$cat</TD>\n";
		print "	<TD class=Odd align=right>$val</TD>\n";
		print "</TR>\n";
		next;
	}

	# Show data of a run.
	# Format of the input is:
	#<date>:<time>:result::different:<count>:<runtime>:<remark>

	my @field_l = split( ":", $_ );
	if ( $field_l[ 4 ] eq "different" )
	{
		# Make lines for runs with differences emphasised.
		print "	<TR class=" . $c[ $c_i++ % 2 ] . "_Emp>\n";

		# When there are differences, we want to go back to this point in the table
		print "<TD class=Even><A NAME=\"back_" . $field_l[ 0 ] . "_" . $field_l[ 1 ] . "\"></A>" . $field_l[ 0 ] . "</TD>\n";	# Date
	}
	else
	{
		print "	<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
		print "	<TD class=Even>" . $field_l[ 0 ] . "</TD>\n";	# Date
	}
	print "	<TD class=Odd>" . $field_l[ 1 ] . "</TD>\n";		# Time
	if ( ( 0 + $field_l[ 6 ] ) > 0 )			# Runtime available?
	{
		print "	<TD class=Even align=right>$field_l[ 6 ]</TD>\n";
	}
	else
	{
		print "	<TD class=Even>&nbsp;</TD>\n";
	}
	if ( $field_l[ 4 ] eq "different" )		# Result
	{
		print "	<TD class=Odd_Emp><A HREF=\"#log_" . $field_l[ 0 ] . "_" . $field_l[ 1 ] . "\">" . $field_l[ 4 ] . "</A></TD>\n";
	}
	else
	{
		print "	<TD class=Odd>" . $field_l[ 4 ] . "</TD>\n";
	}
	if ( $field_l[ 5 ] )			# Count available?
	{
		print "	<TD class=Even align=right>$field_l[ 5 ]</TD>\n";
	}
	else
	{
		print "	<TD class=Even>&nbsp;</TD>\n";
	}
	my $remark = $field_l[ 7 ];				# Possibly contains ":"
	for ( my $i = 8; $field_l[ $i + 1 ]; $i++ )
	{
		$remark = sprintf( "%s: %s", $remark, $field_l[ $i ] );
	}
	if ( $remark )
	{
		print "	<TD class=Odd_Emp align=left>$remark</TD>\n";
	}
	else
	{
		print "	<TD class=Odd>&nbsp;</TD>\n";
	}
	print "</TR>\n";
}
close STDIN;

print "</TBODY>\n";
print "</TABLE>\n";
print "</DIV><!-- class=SCC_LOG_STATS -->\n";

# Now the table is present, we show the data.
# Use the date/time of the run as the ID for HREF and NAME.
my $line_cnt = 1;
open STDIN,"<$tmp_1_file" || die "Could not open temporary-file";
open TMP_2,">$tmp_2_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	chomp;
	my @field_l = split( ":", $_ );
	$field_l[ 0 ] =~ s/-//g;
	$field_l[ 1 ] =~ s/\.//g;
	print TMP_2 $field_l[ 0 ] . ":" . $field_l[ 1 ] . ":" . $line_cnt . ":" . $_ . "\n";
	$line_cnt++;
}
close STDIN;
close TMP_2;

@args =	(	
			"\"$sort_exe\"",
			"-t", ":",
			"-k", "1,1nr",
			"-k", "2,2nr",
			"-k", "3,3n",
			"-o", "$tmp_3_file",
			"$tmp_2_file"
		);
if ( system( @args ) )
{
	print STDERR "$PROGNAME: unable to sort statistics: $! \n";
	exit 1;				# Indicate failure.
}

# Show the data for runs that produced differences.
my $first_log_entry = 1;
my $show_data = 0;
my $prev_class = "";
open STDIN,"<$tmp_3_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	chomp;
	s/^[0-9]*://;		# erase date that was added for sorting
	s/^[0-9]*://;		# erase time that was added for sorting
	s/^[0-9]*://;		# erase record-nr that was added for sorting

	if ( $_ =~ /:result::different$/ )
	{
		my @field_l = split( ":", $_ );

		if ( $show_data > 0 )
		{
			print "</PRE>\n";		# end of previously showed data
		}
		$show_data  = 1;			# indicate that we are going to show differences
		$prev_class = "";			# reset class-identifier for this run
		print "<HR>\n";
		if ( $first_log_entry )
		{
			print "<DIV class=SCC_LOG_ENTRY>\n";
			$first_log_entry = 0;
		}
		print "<H4>$TOP_URL<A HREF=\"#back_$field_l[ 0 ]_$field_l[ 1 ]\">Back</A>&nbsp;&nbsp;&nbsp;&nbsp;";
		print "<A NAME=\"log_$field_l[ 0 ]_$field_l[ 1 ]\">Differences at: $field_l[ 0 ] $field_l[ 1 ]</A></H4>\n";
		print "<PRE>\n";
		next;
	}

	if ( $_ =~ /:result::identical$/ || $_ =~ /:result::.re.start$/ )
	{
		if ( $show_data > 0 )
		{
			print "</PRE>\n";		# end of previously showed data
		}
		$show_data  = 0;			# there are no differences: do not show data
		next;
	}

	if ( $_ =~ /:data::/ )
	{
		if ( $show_data > 0 )
		{
			# Each line contains the following fields:
			# <date>:<time>:data::old|new::<classification>::<data>

			# Remove <date>:<time>:data::
			s/::/ScC_MaRkEr::/;
			s/.*ScC_MaRkEr:://;

			my $change = $_;
			$change =~ s/:.*//;			# old/new

			# Remove old|new::
			s/::/ScC_MaRkEr::/;
			s/.*ScC_MaRkEr:://;

			my $class = $_;
			$class =~ s/::.*//;			# remove the data
			$class =~ s/:/ - /g;			# remove the data

			# Do not show the classification on each line, only once in bold format.
			if ( $class ne $prev_class )
			{
				print "<BR>\n";
				print "<B>" . $class  ."</B>\n";
				$prev_class = $class;
			}

			# Remove the classification
			s/::/ScC_MaRkEr::/;
			s/.*ScC_MaRkEr:://;

			print $change . ":	" . $_ . "\n";
			next;
		}
	}

	# No data, classifications:
	# possible classifications are: count, remark, previous date and time
	if ( $show_data > 0 )
	{
		my @field_l = split( ":", $_ );
		printf( "%-30.30s: %s\n", $field_l[ 2 ], $field_l[ 4 ] );
	}
	next;
}
close STDIN;

if ( $show_data > 0 )
{
	print "</PRE>\n";		# end of previously showed data
}

print "<HR>\n";
if ( $first_log_entry < 1 )
{
	print "</DIV><!-- class=SCC_LOG_ENTRY -->\n";
}

my $date_string = scalar localtime;
print "<P>Generated by SCC (&copy; <A HREF=\"http://scc.qnh-infrastructure.nl\">QNH</A>) on $date_string</P>\n";

print "</DIV><!-- class=SCC_LOG -->\n";
print "</BODY>\n";
print "</HTML>\n";

unlink $tmp_1_file;
unlink $tmp_2_file;
unlink $tmp_3_file;

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

exit 0

