# To be called with full path of this script!

use strict "vars";

# Perl script to convert a SCC snapshot to html-format.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: http://scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.25 $


my $PROGNAME = "$0";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME \n";
}

my $cmd;

my $data;
my $url;
my $tag;
my $sep;
my $class;
my $main_cnt;

my $tmp_1_file = "snap2html_1";
my $tmp_2_file = "snap2html_2";
my $tmp_3_file = "snap2html_3";
my $tmp_4_file = "snap2html_4";

my $host="$ARGV[0]";

my $color = "purple";

my $TOP_NAME="<A NAME=cfg_top></A>";
my $TOP_URL="<A HREF=\"#cfg_top\">Top</A>&nbsp;&nbsp;&nbsp;&nbsp;";

print "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
print "<HTML>\n";

print "<HEAD>\n";
print "<LINK HREF=\"style.css\" REL=\"stylesheet\" TYPE=\"text/css\">\n";
print "<TITLE>Configuration: $host</TITLE>\n";
print "</HEAD>\n";

print "<BODY>\n";
print "<DIV class=SCC_SNAP>\n";
print "$TOP_NAME\n";
print "<H1>\n";
print "	Configuration: $host\n";
print "</H1>\n";

print "<DIV class=SCC_SNAP_NAV>\n";

# DO NOT CHANGE THE FOLLOWING HTML-CODE WITHOUT CONSULTING scc.cgi IN scc-srv
print "<H4>\n";
print "	<A HREF=\"index.html\">Home</A>&nbsp;&nbsp;&nbsp;&nbsp;\n";
print "	<A HREF=\"scc.$host.log.html\">Logbook</A>\n";
print "	&nbsp;&nbsp;&nbsp;&nbsp;<A HREF=\"#statistics\">Statistics</A>\n";
print "	&nbsp;&nbsp;&nbsp;&nbsp;<A HREF=\"#legend\">Legend</A>\n";
print "</H4>\n";

print "</DIV><!-- class=SCC_SNAP_NAV -->\n";

open TMP_1,">$tmp_1_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	if ( $_ =~ /^var:MoDuLe:/ )
	{
		next;
	}

	s/&/\&amp;/g;
	s/</\&lt;/g;
	s/>/\&gt;/g;
	s/'/\&#39;/g;
	s/"/\&quot;/g;

	if ( $_ =~ /^fix:/ || $_ =~ /^var:/ || $_ =~ /^hlp:/ )
	{
		print TMP_1 $_;
	}
	else
	{
		print TMP_1 "fix:unexpected data::$_";
	}

}
close STDIN;
close TMP_1;

# Get all the help classifications from the snap-shot.
open STDIN,"<$tmp_1_file";
my %help_available = ();
while ( <STDIN> )
{
	if ( $_ =~ /^hlp:/ )
	{
		chomp;

		s/::.*//;
		s/^hlp://;
		s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
		
		$help_available{ $_ } = 1;
	}
}
close STDIN;

# Get all the hierarchical classifications from the snap-shot.
# Do not sort, we depend on the order of the collect-script.
my $prev_line = "";
my $line_cnt = 0;
my %id_l = ();
open STDIN,"<$tmp_1_file";
open TMP_2,">$tmp_2_file" || die "Could not open temporary-file";
while ( <STDIN> )
{
	if ( $_ =~ /^fix:/ || $_ =~ /^var:/ )
	{
		s/^var://;
		s/^fix://;
		s/::.*//;
		s/\\/_/g;

		if ( $_ ne $prev_line )
		{
			$line_cnt += 1;
			$prev_line = $_;

			# Format of the input is (for level 3 classification):
			# a:b:c
			# Generate:
			# - level 1 menu item: a
			# - level 2 menu item: a:b
			# - level 3 menu item: a:b:c
			# Each line of output has the following format:
			# <level>:<id of parent menu>:<id of menu-item>:<classification>

			my $prev_class;
			my $level = 1;
			chomp;
			my @field_l = split( ":", $_ );
			foreach my $field (@field_l)
			{
				$prev_class = $class;
				if ( $level eq 1 )
				{
					$class = $field;
				}
				else
				{
					$class = sprintf( "%s:%s", $class, $field );
				}
				if ( ( $id_l{ $class } + 0 ) eq 0 )
				{
					# One line may cause several new submenus.
					$id_l{ $class } = ( 10 * $line_cnt ) + $level;
					print TMP_2 $level . ":" . ( 0 + $id_l{ $prev_class } ) . ":" . $line_cnt . ":" . $class . "\n";
				}
				$level += 1;
			}

		}
	}
}
close STDIN;
close TMP_2;

my $sort_exe = "$ENV{ SCC_BIN }" . "sort";
my @args =	(
			"\"$sort_exe\"",
				"-t", ":",
				"-k", "1n",
				"-k", "2n",
				"-k", "3n",
				"-o", "$tmp_4_file",
				"$tmp_2_file"
		);
if ( system( @args ) )
{
	print STDERR "$PROGNAME: unable to sort classifications: $! \n";
	exit 1;				# Indicate failure.
}

# Now we have all menu data sorted in the right order. Produce the menu.
print "<DIV class=SCC_SNAP_MENU>\n";
my $prev_level = "-1";
my $prev_menu = "-1";
my $label = "";
open STDIN,"<$tmp_4_file";
my %help_used = ();
while ( <STDIN> )
{
	chomp;

	# Format of the data is:
	# <level>:<submenu-id>:<line-number>:<classification>

	my $i;
	my @field_l = split( ":", $_ );
	my $level = $field_l[ 0 ];
	if ( $level != $prev_level )
	{
		if ( $prev_level )
		{
			print "<HR>\n";
		}
		$prev_level = $level;
	}

	if ( ( $level > 1 ) && ( $field_l[ 1 ] ne $prev_menu ) )
	{
		# Produce the heading of a submenu (name, top-reference, optional help, label).
		print "<H4>\n";

		# Produce the name of this submenu.
		$url = $field_l[ 3 ];
		for ( $i = 4; $field_l[ ( $i + 1 ) ]; $i++ )
		{
			$url = sprintf( "%s_%s", $url, $field_l[ $i ] );
		}
		$url =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
		$url =~ s/_$//;
		print "	<A NAME=\"cfg_$url\"></A>\n";

		print "	" . $TOP_URL . "\n";			# Reference to TOP

		if ( ( $help_available{ $url } + 0 ) > 0 )
		{
			print "	<A HREF=\"#hlp_$url\">Help</A>&nbsp;&nbsp;&nbsp;&nbsp;";
			$help_used{ $url } = 1;	# Indicate that this help-info is referenced.
		}

		# Produce references to parent menus.
		$url = "cfg";
		for ( $i = 3; $field_l[ ( $i + 2 ) ] ; $i++ )
		{
			$url = sprintf( "%s_%s", $url, $field_l[ $i ] );
			$url =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
			printf( "	<A HREF=\"#%s\">%s</A> - \n", $url, $field_l[ $i ] );
		}
		print "	" . $field_l[ $i ] . "\n";

		print "</H4>\n";
		$prev_menu = $field_l[ 1 ];
	}

	$url = $field_l[ 3 ];
	$label = $url;
	for ( $i = 4; $field_l[ $i ]; $i++ )
	{
		$url = sprintf( "%s_%s", $url, $field_l[ $i ] );
		$label = $field_l[ $i ];
	}
	$url =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
	printf( "		<A HREF=\"#cfg_%s\">%s</A><BR>\n", $url, $label );
}
close STDIN;
print "</DIV><!-- class=SCC_SNAP_MENU -->\n";

# The entire menu-structure has been built.
# Show all the data.
# To "group" all data per classification, we rewrite the snapshot.
# - each "new" classification is prefixed by the line:
#	<first-line-number>:0:<class>
# - each line of a snapshot is extended with:
#	<first-line-number-of-classification>:<line-number>:<data>
# By sorting the resulting file, all data with the same classifications are
# grouped together, while the order of the lines within a classification
# is preserved from the original snapshot.
open STDIN,"<$tmp_1_file";
open TMP_2,">$tmp_2_file";
%id_l = ();
$line_cnt = 1;
while ( <STDIN> )
{
	chomp;

	if ( $_ =~ /^fix:/ || $_ =~ /^var:/ )
	{
		$class = $_;
		$class =~ s/^var://;
		$class =~ s/^fix://;
		$class =~ s/::.*//;
		$label = $class;
		$label =~ s/:/ - /g;

		if ( ( $id_l{ $class } + 0 ) eq 0 )
		{
			# Indicate the start of a new classification.
			# Format is: <id>:<0>:<header of classification>:<classification>
			$id_l{ $class } = $line_cnt;
			print TMP_2 $line_cnt . ":0:" . $label . ":" . $class . "\n";
		}
		print TMP_2 $id_l{ $class } . ":" . $line_cnt . ":" . $_ . "\n";
		$line_cnt++;
	}
}
close TMP_2;
close STDIN;

my $sort_exe = "$ENV{ SCC_BIN }" . "sort";
my @args =	(
			"\"$sort_exe\"",
				"-t", ":",
				"-k", "1n",
				"-k", "2n",
				"-o", "$tmp_3_file",
				"$tmp_2_file"
		);
if ( system( @args ) )
{
	print STDERR "$PROGNAME: unable to sort SCC-data: $! \n";
	exit 1;				# Indicate failure.
}

# Now all data is sorted in the right order. Show all data (no help-info yet).
print "<DIV class=SCC_SNAP_ENTRY>\n";
my $started = 0;
my $general = 0;
$url = "";
my $i = 0;
my $var_font_start = "";
my $var_font_end = "";
open STDIN,"<$tmp_3_file";
while ( <STDIN> )
{
	chomp;

	my @field_l = split( ":", $_ );
	if ( ( $field_l[ 1 ] + 0 ) eq 0 )
	{
		# This is the start of a new classification.
		# Format is: <id>:<0>:<header of classification>:<classification>
		if ( $started > 0 )
		{
			print "</PRE>\n";
			$general = 0;
		}
		$started = 1;
		print "<HR>\n";

		print "<H4>" . $TOP_URL;

		$url = $_;
		$url =~ s/^[0-9]*:0:[^:][^:]*://;		# Only keep the classification
		$url =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".

		if ( ( $help_available{ $url } + 0 ) > 0 )
		{
			print "<A HREF=\"#hlp_$url\">Help</A>&nbsp;&nbsp;&nbsp;&nbsp;";
			$help_used{ $url } = 1;	# Indicate that this help-info is referenced.
		}

		printf( "<A NAME=\"cfg_%s\"></A>", $url );	# Produce the name of this sub-menu/data

		# References to all previous menus in the hierarchy.
		$tag = "";
		$sep = "";
		for ( $i = 3; $field_l[ ( $i + 1 ) ]; $i++ )
		{
			$tag = sprintf( "%s_%s", $tag, $field_l[ $i ] );
			$tag =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
			printf( "%s<A HREF=\"#cfg%s\">%s</A>\n", $sep, $tag, $field_l[ $i ] );
			$sep = " - ";
		}
		print "	" . $sep . $field_l[ $i ] . "\n";

		print "</H4>\n";
		print "<PRE>\n";
		if ( $field_l[ 3 ] eq "general" )
		{
			$general = 1;
		}

		next;
	}

	# Show var-data in a different color.
	$var_font_start = "";
	$var_font_end = "";
	if ( ( $field_l[ 2 ] ne "fix" ) && ( $color ne "no_color" ) )
	{
		$var_font_start = "<span class=\"scc_var\">";
		$var_font_end = "</span>";
	}

	if ( $general > 0 )
	{
		# Format of the data is:
		#<classID>:<lineNR>:fix_or_var:general::<label>:<data>
		#0         1        2          3       4 5      6
		print $var_font_start;
		$data = $_;
		$data =~ s/.*:general:://;
		$label = $data;
		$label =~ s/:.*//;
		$data =~ s/^[^:][^:]*://;

		printf( "%-25s: %s%s\n", $label, $data, $var_font_end );
	}
	else
	{
		# Format of the data is:
		#<classID>:<lineNR>:fix_or_var:<class>::<data>
		#0         1        2          3        
		# We only want to display the data, ignore leading fields.
		$data = $_;
		# Avoid mutilation of lines containing "::"
		$data =~ s/::/ScC_MaRkEr::/;
		$data =~ s/.*ScC_MaRkEr:://;
		if ( $data )
		{
			print $var_font_start . $data . $var_font_end . "\n";
		}
		else
		{
			print "&nbsp;\n";
		}
	}
}
print "</PRE>\n";
close STDIN;
print "</DIV><!-- class=SCC_SNAP_ENTRY -->\n";

# Calculate and show the statistics.
print "<DIV class=SCC_SNAP_STATS>\n";
print "<HR>\n";
print "<H4>${TOP_URL}<A NAME=statistics>Statistics</A></H4>\n";

print "<P>Help-info is excluded from the statistics.</P>\n";

open STDIN,"<$tmp_1_file";
my %main_cnt = ();
my %sub_cnt = ();
my $fix_cnt = 0;
my $var_cnt = 0;
my $total = 0;
open TMP_2,">$tmp_2_file";
while ( <STDIN> )
{
	chomp;

	if ( $_ =~ /^fix:/ )
	{
		$fix_cnt++;
	}
	if ( $_ =~ /^var:/ )
	{
		$var_cnt++;
	}
	my @field_l = split( ":", $_ );
	$main_cnt{ $field_l[ 1 ] }++;
	$class = sprintf( "%s:%s", $field_l[ 1 ], $field_l[ 2 ] );
	$sub_cnt{ $class }++;

	$total++;
}
close STDIN;

@$main_cnt{ "general:" } += 4;		# start/stop/runtime added by scc-log
$total += 4;				# start/stop/runtime added by scc-log
$var_cnt += 4;				# start/stop/runtime added by scc-log


my $fix_perc = sprintf "%d", ( 0.5 + ( 100 * $fix_cnt ) / $total );
my $var_perc = sprintf "%d", ( 0.5 + ( 100 * $var_cnt ) / $total );

print "<TABLE CLASS=SCC SUMMARY=fix_var_counts BORDER CELLSPACING=1 CELLPADDING=1>\n";
print "<THEAD>\n";
print "<TR class=Odd>\n";
print "	<TH>Category</TH>\n";
print "	<TH>Count</TH>\n";
print "	<TH>Percentage</TH>\n";
print "</TR>\n";
print "</THEAD>\n";

print "<TBODY>\n";

print "<TR class=Even>\n";
print "	<TD class=Odd>fix</TD>\n";
print "	<TD class=Even align=right>" . $fix_cnt . "</TD>\n";
print "	<TD class=Odd align=right>" . $fix_perc . "</TD>\n";
print "</TR>\n";

print "<TR class=Odd>\n";
print "	<TD class=Odd>var</TD>\n";
print "	<TD class=Even align=right>" . $var_cnt . "</TD>\n";
print "	<TD class=Odd align=right>" . $var_perc . "</TD>\n";
print "</TR>\n";
print "<TR class=Even>\n";
print "	<TD class=Odd>total</TD>\n";
print "	<TD class=Even align=right>" . $total . "</TD>\n";
print "	<TD class=Odd align=right>" . "100" . "</TD>\n";
print "</TR>\n";
print "</TBODY>\n";
print "</TABLE>\n";

my $value;
while ( ( $class, $value ) = each (%sub_cnt) )
{
	my $main_class = $class;
	$main_class =~ s/:.*//;

	my $main_counter = $main_cnt{ $main_class };

	my $main_perc = sprintf "%d", ( 0.5 + ( 100 * $main_counter ) / $total );
	my $sub_perc = sprintf "%d", ( 0.5 + ( 100 * $value ) / $main_counter );

	print TMP_2 $class . ":" . $main_counter . ":" . $main_perc . ":" . $value . ":" . $sub_perc . "\n";
}
close TMP_2;

my $sort_exe = "$ENV{ SCC_BIN }" . "sort";
my @args =	(
			"\"$sort_exe\"",
				"-t", ":",
				"-k", "3nr",
				"-k", "1,1",
				"-k", "5nr",
				"-o", "$tmp_3_file",
				"$tmp_2_file"
		);
if ( system( @args ) )
{
	print STDERR "$PROGNAME: unable to sort statistics: $! \n";
	exit 1;				# Indicate failure.
}

my @c = ( "Even", "Odd" );
my $c_i = 1;

print "<BR>\n";
print "<TABLE CLASS=SCC SUMMARY=class_counts BORDER CELLSPACING=1 CELLPADDING=1>\n";
print "<THEAD>\n";
print "<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
print "	<TH>Main class</TH>\n";
print "	<TH>Sub class</TH>\n";
print "	<TH>Main cnt</TH>\n";
print "	<TH>Main perc</TH>\n";
print "	<TH>Sub cnt</TH>\n";
print "	<TH>Sub perc</TH>\n";
print "</TR>\n";
print "</THEAD>\n";

print "<TBODY>\n";

my $prev = "";
open STDIN,"<$tmp_3_file";
while ( <STDIN> )
{
	chomp;

	$class = $_;
	$class =~ s/:.*//;

	my @field_l = split( ":", $_ );

	if ( $field_l[ 0 ] ne $prev )
	{
		# Print the main-count
		my $url = "cfg_" . $field_l[ 0 ];
		$url =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
		print "<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
		print "	<TD class=Odd><A HREF=\"#" . $url . "\">" . $field_l[ 0 ] . "</A></TD>\n";
		print "	<TD class=Even> &nbsp; </TD>\n";
		print "	<TD class=Odd align=right>" . $field_l[ 2 ] . "</TD>\n";
		print "	<TD class=Even align=right>" . $field_l[ 3 ] . "</TD>\n";
		print "	<TD class=Odd> &nbsp; </TD>\n";
		print "	<TD class=Even> &nbsp; </TD>\n";
		print "</TR>\n";

		$prev = $field_l[ 0 ];
	}

	if ( $field_l[ 1 ] )
	{
		# Print the sub-count
		my $url = "cfg_" . $prev . "_" . $field_l[ 1 ];
		$url =~ s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
		print "<TR class=" . $c[ $c_i++ % 2 ] . ">\n";
		print "	<TD class=Odd> &nbsp; </TD>\n";
		print "	<TD class=Even><A HREF=\"#" . $url . "\">" . $field_l[ 1 ] . "</A></TD>\n";
		print "	<TD class=Odd> &nbsp; </TD>\n";
		print "	<TD class=Even> &nbsp; </TD>\n";
		print "	<TD class=Odd align=right>" . $field_l[ 4 ] .  "</TD>\n";
		print "	<TD class=Even align=right>" . $field_l[ 5 ] .  "</TD>\n";
		print "</TR>\n"
	}
}
close STDIN;

print "</TBODY>\n";
print "</TABLE>\n";
print "</DIV><!-- class=SCC_SNAP_STATS -->\n";

# Show all the help-info
# Determine the available classes.
print "<DIV class=SCC_SNAP_HELP>\n";
$prev = "";
open STDIN,"<$tmp_1_file";
while ( <STDIN> )
{
	if ( $_ =~ /^hlp:/ )
	{
		chomp;
		$data = $_;
		s/::.*//;
		s/^hlp://;
		$class = $_;
		s/[^A-Za-z0-9_.-]/_/g;			# W3C restrictions and no ":".
		if ( ( 0 + $help_used{ $_ } ) > 0 )
		{
			if ( $_ ne $prev )
			{
				if ( $prev )
				{
					print "</PRE>\n";
					# Some config files can be checked more than once.
					# Show the helpinfo only once to avoid double definition of NAME entry.
					$help_used{ $prev } = 0;
				}
				$prev = $_;

				print "<HR>\n";
				print "<A NAME=\"hlp_$_\"></A>\n";

				print "<H4>$TOP_URL";

				$url = $_;
				$url =~ s/^/cfg_/;

				print "<A HREF=\"#$url\">Back</A>&nbsp;&nbsp;&nbsp;&nbsp;";

				$class =~ s/:/ - /g;
				print $class;
				print "</H4>\n";
				print "<PRE>\n";
			}

			# Avoid mutilation of lines containing "::"
			$data =~ s/::/ScC_MaRkEr::/;
			$data =~ s/.*ScC_MaRkEr:://;
			if ( $data )
			{
				print $data . "\n";
			}
			else
			{
				print "&nbsp;\n";
			}
		}
	}
}
close STDIN;

if ( $prev )
{
	print "</PRE>\n";
}
print "</DIV><!-- class=SCC_SNAP_HELP -->\n";

print "<HR>\n";
print "<H4>$TOP_URL<A NAME=legend>Legend</A></H4>\n";
print "<P>\n";
print "Fix data in the snapshot has a default color.\n";
print "Variable data in the snapshot has a specific <span class=\"scc_var\">color</span>.\n";
print "</P>\n";

print "<HR>\n";
my $date_string = scalar localtime;
print "<P>Generated by SCC (&copy; <A HREF=\"http://scc.qnh-infrastructure.nl\">QNH</A>) on $date_string</P>\n";

print "</DIV><!-- class=SCC_SNAP -->\n";

print "</BODY>\n";
print "</HTML>\n";

unlink $tmp_1_file;
unlink $tmp_2_file;
unlink $tmp_3_file;
unlink $tmp_4_file;

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

exit 0;
