# To be called with full path of this script!

use strict "vars";

# Perl script to collect SCC data from current (and other) system(s).
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: http://scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.42 $


my $PROGNAME = "$0";

my $SYNTAX = "Syntax error, use:
	$PROGNAME	[ --file <file> ] [ --list <list> ] [ --netview ]
		[ --missing ] [ --min_age <hours> ]
		[ --delay <delay> ] [ --extra <mod> ] [ --norun ]
		[ --from <mail> ] [ --dest <dest> ] [ --passwd <pw> ] [ --pki <file> ]
		[ --prog <prog> ] [ --smtp <smtp> ] [ --trace ] [ --report <file> ]
		[ --timeout <duration> ] [ --help ] [ --version ]\n";

use Cwd;

# Determine our directories. Assume this script is called in the bin-directory.
my $org_dir = getcwd();
my $dir = $org_dir;
$dir =~ s/\//\\/g;
$dir =~ s/bin$//;

my $SCC_DATA	= $dir		. "data\\";
my $SCC_TMP	= $dir		. "tmp\\";
my $SCC_BIN	= $dir		. "bin\\";
my $SCC_MODULES	= $SCC_BIN	. "scc_modules\\";

$ENV{ SCC_BIN } = $SCC_BIN;
$ENV{ SCC_TMP } = $SCC_TMP;
$ENV{ SCC_DATA } = $SCC_DATA;

mkdir( $SCC_TMP, 0777 );
mkdir( $SCC_DATA, 0777 );
mkdir( $SCC_DATA . "transfer", 0777 );

my $tmp_1_file = $SCC_TMP . "scc_1.txt";
my $tmp_2_file = $SCC_TMP . "scc_2.txt";
my $tmp_3_file = $SCC_TMP . "scc_3.txt";

close STDOUT;
open STDOUT,">$tmp_2_file";
close STDOUT;
close STDIN;
open STDIN,"<$tmp_2_file";		# reopen STDIN immediately to avoid later warnings.

my $index_html = $SCC_DATA . "index.html";

sub	get_fqdn
{
	my $hostname = "\L$_[ 0 ]";

	if ( system( "nslookup $hostname >\"$tmp_3_file\"" ) )
	{
		print STDERR "$PROGNAME: unable to call nslookup to determine FQDN: $! \n";
		return( "" );				# Indicate failure.
	}

	my $host_fqdn = "";
	open INPUT,"<$tmp_3_file";
	while ( <INPUT> )
	{
		chomp;
		if ( $_ =~ /^Name/ )
		{
			s/^Name:[ 	]*//;
			$host_fqdn = $_;
		}
	}
	close INPUT;
	unlink( $tmp_3_file );

	return( $host_fqdn );
}

# Handle all options and their arguments.
my $i = 0;
my $local	= 1;
my $run		= 1;			# Do run by default
my $delay	= 0;
my $extra_mod	= "";
my $file	= "";
my $fqdn	= "";
my $netview	= 0;
my $list	= "";
my $min_age	= 0;
my $missing	= "";
my $pki_file	= "";
my $prog	= "mail";
my $dest	= "";
my $from	= "";
my $smtp	= "";
my $passwd	= "";
my $timeout	= "";
my $report_file	= "do_not_keep";
my $trace	= 0;

while ( @ARGV )
{
	if ( "$ARGV[ 0 ]" =~ /^--delay$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$delay = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--dest$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$dest = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--extra$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$extra_mod = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--fqdn$/ )
	{
		$fqdn = "yes";
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--from$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$from = "$ARGV[ 0 ]";
			if ( "$from" =~ /^[^<].*[^>]$/ )
			{
				$from = "<$from>"
			}
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--file$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$local = 0;
			$file = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--help$/ )
	{
		print STDERR "$SYNTAX";
		exit 0;
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--list$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$local = 0;
			$list = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--min_age$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$min_age = $ARGV[ 0 ];
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--missing$/ )
	{
		$missing = "yes";
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--netview/ )
	{
		$local = 0;
		$netview = 1;
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--norun$/ )
	{
		$run	= 0;
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--passwd$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$passwd = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--pki$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$pki_file =  "$SCC_BIN" . "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--prog$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$prog = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--report$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$report_file = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--smtp$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$smtp = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--timeout$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$timeout = "$ARGV[ 0 ]";
		}
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--trace/ )
	{
		$trace = 1;
		print STDERR "trace start in $PROGNAME \n";
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--version/ )
	{
		print STDERR "$PROGNAME: version 1.1.5 \n";
		exit 0 ;
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--/ )
	{
		die "$SYNTAX";
	}
	shift( @ARGV );
}

$ENV{ TRACE } = $trace;

if ( ( $prog =~ /^mail$/ ) && ( $smtp =~ /^$/ ) && ( ! ( $dest =~ /^$/ ) ) )
{
	print STDERR "$PROGNAME: option --prog mail requires --smtp option\n";
	exit 1;
}

if ( $prog =~ /^mail$/ )
{
	if ( "$dest" =~ /^[^<].*[^>]$/ )
	{
		$dest = "<$dest>"
	}
}

my $sys_cnt = 0;
my @sys_list;

if ( $local )				# Process local system only? 
{
	use Win32::OLE('in');

	use constant wbemFlagReturnImmediately => 0x10;
	use constant wbemFlagForwardOnly => 0x20;

	my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\.\\root\\CIMV2") or die "$PROGNAME: WMI connection failed.\n";
	my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_OperatingSystem", "WQL",
			  wbemFlagReturnImmediately | wbemFlagForwardOnly);

	foreach my $objItem (in $colItems)
	{
		$sys_list[ $sys_cnt ] = "\L$objItem->{CSName}";
		$sys_cnt += 1;
	}
}

if ( $file )				# Process systems named in a file?
{
	open INPUT,"<$file" or die "$PROGNAME: Unable to open file $file ";
	while ( <INPUT> )
	{
		chomp;
		if ( $_ =~ "^\$" )
		{
			next;		# Ignore empty lines.
		}
		if ( $_ =~ "^#" )
		{
			next;		# Ignore comment-lines.
		}
		s/[ 	].*//;
		$sys_list[ $sys_cnt ] = "\L$_";
		$sys_cnt += 1;
	}
	close INPUT;
}

if ( $netview )				# Process all systems from the output of the net view command?
{
	if ( $run )
	{
		if ( system( "net view>\"$tmp_1_file\"" ) )
		{
			print STDERR "$PROGNAME: unable to call net view: $! \n";
			exit 1;				# Indicate failure.
		}
		open INPUT,"<$tmp_1_file";
		while ( <INPUT> )
		{
			chomp;
			if ( $_ =~ /^\\\\/ )
			{
				s/^\\\\//;
				s/[ 	].*//;
				$sys_list[ $sys_cnt ] = "\L$_";
				$sys_cnt += 1;
			}
		}
		close INPUT;
	}
	else
	{
		# Determine all hostnames from the snapshots in the data directory.
		opendir( DIR, "$SCC_DATA" );
		my @files = grep( /^scc\..*\.cur$/, readdir( DIR ) );
		closedir( DIR );

		foreach $file ( @files )
		{
			$file =~ s/^scc\.//; 		# Retrieve the hostname from the filename-format
			$file =~ s/\.cur$//; 
			$sys_list[ $sys_cnt ] = "\L$file";
			$sys_cnt += 1;
		}
	}
}

if ( $list )					# Process systems in a list with names?
{
	my @tmp_sys_list = split( ",", $list );
	foreach my $sys (@tmp_sys_list)
	{
		$sys_list[ $sys_cnt ] = "\L$sys";
		$sys_cnt += 1;
	}
}

if ( "$prog" =~ "^mail" )
{
	use Win32::OLE('in');

	# Determine our domain to create a fixed email addres.
	my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\.\\root\\CIMV2") or die "$PROGNAME: WMI connection failed.\n";
	my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_ComputerSystem", "WQL",
			  wbemFlagReturnImmediately | wbemFlagForwardOnly);

	my $domain = "";
	foreach my $objItem (in $colItems)
	{
		$domain = "$objItem->{Domain}";
	}

	if ( "$from" =~ /^$/ )
	{
		$from = "<postmaster\@$domain>";
	}
}

open REPORT,">$report_file";		# Do this BEFORE the following chdir!

chdir( $SCC_TMP ) or die "$PROGNAME: Cannot change directory to $SCC_TMP";

# Empty logfile that records email transfers.
my $blat_log = $SCC_DATA . "blat_log.txt";
unlink $blat_log;

open INDEX,">$tmp_1_file";
print INDEX "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
print INDEX "<HTML>\n";
print INDEX "	<HEAD>\n";
print INDEX "		<LINK HREF=\"style.css\" REL=\"stylesheet\" TYPE=\"text/css\">\n";
print INDEX "		<TITLE>Index of scc</TITLE>\n";
print INDEX "	</HEAD>\n";
print INDEX "	<BODY>\n";

my $host_in_html = "";
for ( my $i = 0; $i < $sys_cnt; $i++ )
{
	my $hostname = $sys_list[ $i ];

	chdir( $SCC_TMP );

	if ( $run && ( $fqdn =~ /^yes$/ ) )
	{
		$hostname =  get_fqdn( "$hostname" );

		if ( ! $hostname )
		{
			print STDERR "$PROGNAME: unable to collect fqdn for $hostname: $! \n";
			print REPORT "unable to collect data for $hostname \n";
			next;				# skip to next host
		}
	}

	my $snapshot = "$SCC_DATA" . "scc.$hostname.cur";
	my $signal_file = "$SCC_DATA" . "transfer\\scc.$hostname.signal";
	my $skipping = 0;

	if ( -f $snapshot )
	{
		if ( $missing )
		{
			$skipping = 1;
		}
		elsif ( -M $snapshot < $min_age )
		{
			$skipping = 1;
		}
	}

	if ( $skipping )
	{
		print STDERR "Skipping: $hostname \n";
		print REPORT "skipped: $hostname \n";
	}
	else
	{
		print STDERR "Processing: $hostname \n";

		# We should start without signal file to avoid problems in case of --pull.
		# In that case the server will be looking for a signal-file with contents "done".
		unlink $signal_file;

		if ( $run )
		{
			my $log_exe = $SCC_BIN . "scc-log.pl";
			my @args = ( "perl", "\"$log_exe\"" );
			if ( $extra_mod ne "")
			{
				@args = ( @args, "--extra", "$extra_mod" );
			}
			if ( $timeout ne "")
			{
				@args = ( @args, "--timeout", "$timeout" );
			}
			@args = ( @args, "$hostname" );
			if ( system( @args ) )
			{
				print STDERR "$PROGNAME: unable to collect data for $hostname: $! \n";
				print REPORT "unable to collect data for $hostname \n";
				next;				# skip to next host
			}
		}
	}

	print INDEX "		<A HREF=scc.$hostname.html>snapshot of $hostname</A>\n";
	print INDEX "		&nbsp;&nbsp;&nbsp;\n";
	print INDEX "		<A HREF=scc.$hostname.log.html>logbook of $hostname</A>\n";
	print INDEX "	<BR>\n";
	$host_in_html = "yes";

	if ( ! $skipping )
	{
		if ( $dest || ( "$prog" =~ "^pull" ) )
		{
			my $pack_file = "$SCC_DATA" . "transfer\\scc.$hostname.tar.gz";

			# Change directory to allow filenames without directories.
			chdir( $SCC_DATA );

			# Pack the data in a single file, compress with gzip.
			# The order of the files is checked on the server.
			unlink "$pack_file";
			my $tar_exe = $SCC_BIN . "bsdtar";
			my @args =	(
						"\"$tar_exe\"",
							"-c",
							"-z", 
							"-f", "$pack_file",
							"scc.$hostname.cur",
							"scc.$hostname.log",
							"scc.$hostname.log.html",
							"scc.$hostname.html"
					);
			if ( system( @args ) )
			{
				print STDERR "$PROGNAME: unable to pack SCC-data for transport: $! \n";
				print REPORT "unable to pack SCC-data for $hostname \n";
				next;				# Continue with next host
			}

			my $processing_done = 0;

			if ( "$prog" =~ "^mail" )
			{
				# Email receiver at scc-srv expects a file named scc-transfer-data.gz
				rename $pack_file, "scc-transfer-data.gz";

				# Use this commandline tool to mail a file.
				my $blat_exe = $SCC_BIN . "blat";
				my @args =	(
							"\"$blat_exe\"",
								"scc-transfer-data.gz",
								"-log",		"$blat_log",
								"-debug",
								"-uuencode",
								"-f",		"$from",
								"-to",		"$dest",
								"-server",	"$smtp"
						);
				if ( system( @args ) )
				{
					print STDERR "$PROGNAME: unable to mail SCC-data: $! \n";
					print REPORT "unable to mail SCC-data for $hostname \n";
					next;
				}

				unlink "scc-transfer-data.gz";

				$processing_done = 1;
			}

			if ( "$prog" =~ "^pull" )
			{
				# The datafile is already in place.
				# The server is checking for signal files containing one line with: "done".
				open SIGNAL,">$signal_file";
				print SIGNAL "done\n";
				close SIGNAL;

				$processing_done = 1;
			}

			if ( "$prog" =~ "^cp" )
			{
				# The server will be looking for a signal file containing "done".
				# Transfer an empty file to indicate the transfer is in progress.
				open SIGNAL,">$signal_file";
				close SIGNAL;
				if ( system( ( "copy", "\"$signal_file\"", "\"$dest\"" ) ) )
				{
					print STDERR "$PROGNAME: unable to copy $signal_file: $! \n";
					next;
				}

				# Transfer the data-file.
				if ( system( ( "copy", "\"$pack_file\"", "\"$dest\"" ) ) )
				{
					print STDERR "$PROGNAME: unable to copy $pack_file: $! \n";
					next;
				}

				# Signal for the server that the transfer has been completed.
				open SIGNAL,">$signal_file";
				print SIGNAL "done\n";
				close SIGNAL;
				if ( system( ( "copy", "\"$signal_file\"", "\"$dest\"" ) ) )
				{
					print STDERR "$PROGNAME: unable to copy $signal_file: $! \n";
					next;
				}

				$processing_done = 1;
			}

			if ( "$prog" =~ "^scp" )
			{
				# To avoid that the server starts reading a file that has not been transferred
				# completely, we start by sending an empty signal file, followed by the data
				# we want to transfer. Finally we transfer a signal file with contents "done".
				my $transfer = "";
				for ( my $j = 1; $j <= 3; $j++ )
				{
					if ( $j eq 1 )
					{
						$transfer = "scc.$hostname.signal";
						open SIGNAL,">$transfer";
						close SIGNAL;
					}
					if ( $j eq 2 )
					{
						$transfer = "$pack_file";
					}
					if ( $j eq 3 )
					{
						$transfer = "scc.$hostname.signal";
						open SIGNAL,">$transfer";
						print SIGNAL "done\n";
						close SIGNAL;
					}

					my $pscp_exe = $SCC_BIN . "pscp";
					my @args =	(
								"\"$pscp_exe\"",
									"-batch",
									"-q",
							);
					if ( "$passwd" )
					{
						@args = ( @args, "-pw", "$passwd" );
					}
					if ( "$pki_file" )
					{
						@args = ( @args, "-i", "\"$pki_file\"" );
					}
					@args = ( @args, "$transfer", "$dest" );
						
					if ( system( @args ) )
					{
						print STDERR "$PROGNAME: unable to scp $transfer: $! \n";
						print REPORT "unable to scp for $hostname \n";
						next;
					}

					unlink $transfer;
				}	# for ( $j = 1; $j < 4; $j++ )

				$processing_done = 1;
			}	# if ( $prog = "scp" )

			chdir( $SCC_TMP );

			if ( ! $processing_done )
			{
				print STDERR "$PROGNAME: unknown value for --prog option: $prog \n";
				exit 1;				# Indicate failure.
			}
		}	# if ( $dest )
	}	# if ( $skipping )
	print REPORT "Processed: $hostname \n";

	if ( $delay > 0 )
	{
		# Wait a while to ease the system collecting data and (possibly) the receiving system.
		sleep( $delay );	
	}

}	# for ( i= 0; i < $sys_cnt; i++ )

close REPORT;

if ( ! $host_in_html )
{
	print INDEX "<P>All hosts failed when collecting data.</P>\n";
}
print INDEX "	</BODY>\n";
print INDEX "</HTML>\n";
close INDEX;

unlink $index_html;
rename $tmp_1_file, $index_html;

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}
close STDIN;
unlink $tmp_2_file;

chdir( $org_dir );
unlink "do_not_keep";		# This is a no-op when --report option is used.

sleep( 5 );

exit 0;
