# Based on code generated by ScriptoMatic from MicroSoft.

# Perl script to collect harddware data.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.15 $

my $PROGNAME = "$0";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME \n";
}

use strict;

use Win32::OLE('in');

use constant wbemFlagReturnImmediately => 0x10;
use constant wbemFlagForwardOnly => 0x20;

my $computer = $ENV{ SCC_WMI_COMPUTER };

my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2") or die "$PROGNAME: WMI connection failed.\n";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "	start Win32_Processor\n";
}

my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2") or die "WMI connection failed.\n";
my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_Processor", "WQL",
                  wbemFlagReturnImmediately | wbemFlagForwardOnly);

print "hlp:hardware:cpu::Data from class Win32_Processor\n";
foreach my $objItem (in $colItems)
{
	my $label = "hardware:cpu:$objItem->{DeviceID}";

	print "fix:" . $label . "::AddressWidth		: $objItem->{AddressWidth}\n";
	print "var:" . $label . "::CurrentClockSpeed	: $objItem->{CurrentClockSpeed}\n";
	print "fix:" . $label . "::Description		: $objItem->{Description}\n";
	print "fix:" . $label . "::ExtClock		: $objItem->{ExtClock}\n";
	print "fix:" . $label . "::L2CacheSize		: $objItem->{L2CacheSize}\n";
	print "fix:" . $label . "::Manufacturer		: $objItem->{Manufacturer}\n";
	my $name = $objItem->{Name};
	$name =~ s/^[ 	]*//;
	print "fix:" . $label . "::Name			: $name\n";
	print "fix:" . $label . "::ProcessorType		: $objItem->{ProcessorType}\n";
	print "fix:" . $label . "::Revision		: $objItem->{Revision}\n";
	print "fix:" . $label . "::Stepping		: $objItem->{Stepping}\n";
	print "fix:" . $label . "::Version			: $objItem->{Version}\n";
}

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "	start Win32_DiskDrive\n";
}
my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_DiskDrive", "WQL",
			wbemFlagReturnImmediately | wbemFlagForwardOnly);

print "hlp:hardware:diskdrive::Data from class Win32_DiskDrive\n";
foreach my $objItem (in $colItems)
{
	my $fix_label = "fix:hardware:diskdrive::";

	print $fix_label;
	printf( "%-6s", "$objItem->{InterfaceType}" );
	printf( " %6d MB", $objItem->{Size} / ( 1024 * 1024 ) );
	printf( " %-30s", "$objItem->{Model}" );
	printf( " partitions: %.2d", "$objItem->{Partitions}" );

	print " SCSI Bus: $objItem->{SCSIBus}";
	print " LUN: $objItem->{SCSILogicalUnit}";

	# For SAN disks the SCSIPort and SCSITargetId appear to be variable.
	#print " Port: $objItem->{SCSIPort}";
	#print " Target: $objItem->{SCSITargetId}";

	print "\n";
}

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "	start Win32_PhysicalMemory\n";
}

my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_PhysicalMemory", "WQL",
			wbemFlagReturnImmediately | wbemFlagForwardOnly);

# Source: http://msdn2.microsoft.com/en-us/library/aa394347.aspx
my %memory_type = ();
$memory_type{ 0 } = "Standalone Workstation";
$memory_type{ 0 } = "Unknown";
$memory_type{ 1 } = "Other";
$memory_type{ 2 } = "DRAM";
$memory_type{ 3 } = "Synchronous DRAM";
$memory_type{ 4 } = "Cache DRAM";
$memory_type{ 5 } = "EDO";
$memory_type{ 6 } = "EDRAM";
$memory_type{ 7 } = "VRAM";
$memory_type{ 8 } = "SRAM";
$memory_type{ 9 } = "RAM";
$memory_type{ 10 } = "ROM";
$memory_type{ 11 } = "Flash";
$memory_type{ 12 } = "EEPROM";
$memory_type{ 13 } = "FEPROM";
$memory_type{ 14 } = "EPROM";
$memory_type{ 15 } = "CDRAM";
$memory_type{ 16 } = "3DRAM";
$memory_type{ 17 } = "SDRAM";
$memory_type{ 18 } = "SGRAM";
$memory_type{ 19 } = "RDRAM";
$memory_type{ 20 } = "DDR";

# Source: http://msdn2.microsoft.com/en-us/library/aa394347.aspx
my %type_detail = ();
$type_detail{ 1 }	= "Reserved";
$type_detail{ 2 }	= "Other";
$type_detail{ 4 }	= "Unknown";
$type_detail{ 8 }	= "Fast-paged";
$type_detail{ 16 }	= "Static column";
$type_detail{ 32 }	= "Pseudo-static";
$type_detail{ 64 }	= "RAMBUS";
$type_detail{ 128 }	= "Synchronous";
$type_detail{ 256 }	= "CMOS";
$type_detail{ 512 }	= "EDO";
$type_detail{ 1024 }	= "Window DRAM";
$type_detail{ 2048 }	= "Cache DRAM";
$type_detail{ 4096 }	= "Non-volatile";

print "hlp:hardware:memory::Data from class Win32_PhysicalMemory\n";
my $total_memory = 0;
foreach my $objItem (in $colItems)
{
	my $label = "fix:hardware:memory:$objItem->{DeviceLocator}";

	my $capacity = $objItem->{Capacity} / ( 1024 * 1024 ); 
	$total_memory += $capacity;
	print $label . "::Capacity	: $capacity MB\n";
	print $label . "::DataWidth	: $objItem->{DataWidth}\n";
	print $label . "::MemoryType	: $memory_type{ $objItem->{MemoryType} }\n";
	print $label . "::Speed		: $objItem->{Speed}\n";
	print $label . "::TypeDetail	: $type_detail{ $objItem->{TypeDetail} }\n";
}
# This is the hardware module, but we print general data to avoid duplication of code.
print "fix:general::Fysical memory:$total_memory MB\n";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

exit 0;
