# Based on code generated by ScriptoMatic from MicroSoft.

# Perl script to collect volume data.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.16 $

my $PROGNAME = "$0";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME \n";
}

use strict;

use Win32::OLE('in');

use constant wbemFlagReturnImmediately => 0x10;
use constant wbemFlagForwardOnly => 0x20;

my $computer = $ENV{ SCC_WMI_COMPUTER };

my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2") or die "$PROGNAME: WMI connection failed.\n";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "	start Win32_DiskPartition\n";
}
my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_DiskPartition", "WQL",
		wbemFlagReturnImmediately | wbemFlagForwardOnly);

print "hlp:Volume Management:partitions::Data from class Win32_DiskPartition\n";
foreach my $objItem (in $colItems)
{
	my $fix_class = "fix:Volume Management:partitions::";

	print "$fix_class";
	printf( "%-30s", "$objItem->{Name}" );
	printf( "%-30s", "$objItem->{Description}" );
	printf( "%6d MB", $objItem->{Size} / ( 1024 * 1024 ) );
	print "\n";
}

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "	start Win32_LogicalDisk\n";
}
my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_LogicalDisk", "WQL",
			wbemFlagReturnImmediately | wbemFlagForwardOnly);

my %drive_type_list = ();
$drive_type_list{ 0 } = "Unknown";
$drive_type_list{ 1 } = "No Root Directory";
$drive_type_list{ 2 } = "Removable Disk";
$drive_type_list{ 3 } = "Local Disk";
$drive_type_list{ 4 } = "Network Drive";
$drive_type_list{ 5 } = "Compact Disc";
$drive_type_list{ 6 } = "RAM Disk";

my $KB = 1024;
my $MB = 1024 * $KB;
my $GB = 1024 * $MB;

print "hlp:Volume Management:file systems::Data from class Win32_LogicalDisk\n";
my $label = "var:Volume Management:file systems:survey";
printf( "%s::%-6s %-18s %7.7s %7.7s %s\n", $label, "Drive", "Type", "Size", "Free", "Used" );
foreach my $objItem (in $colItems) 
{
	my $perc_used = "";
	my $perc_free = "";
	my $size = $objItem->{Size};
	if ( $size )
	{
		if ( $size < $MB )
		{
			$size = sprintf( "%4.0d KB", $size / $KB );
		}
		elsif ( $size < $GB )
		{
			$size = sprintf( "%4.0d MB", $size / $MB );
		}
		else
		{
			$size = sprintf( "%4.0d GB", $size / $GB );
		}
	}
	my $free = $objItem->{FreeSpace};
	if ( $free )
	{
		if ( $free < $MB )
		{
			$free = sprintf( "%4.0d KB", $free / $KB );
		}
		elsif ( $free < $GB )
		{
			$free = sprintf( "%4.0d MB", $free / $MB );
		}
		else
		{
			$free = sprintf( "%4.0d GB", $free / $GB );
		}
		my $perc = 100 * ( ( $objItem->{Size} - $objItem->{FreeSpace} ) / ( 0 + $objItem->{Size} ) );
		$perc_used = sprintf( "%3.0d%%", ( 0.5 + $perc ) );		# Round, no truncation
		$perc = 100 * ( ( $objItem->{FreeSpace} ) / ( 0 + $objItem->{Size} ) );
		$perc_free = sprintf( "%3.0d%%", ( 0.5 + $perc ) );		# Round, no truncation
	}

	printf( "%s::%-6s %-18s %7s %7s %5s\n",
		$label,
		$objItem->{Name},
		$drive_type_list{ $objItem->{DriveType} },
		$size,
		$free,
		$perc_used );

	my $fix_detail_label = "fix:Volume Management:file systems:$objItem->{Name}";
	my $var_detail_label = $fix_detail_label;
	$var_detail_label =~ s/^fix:/var:/;
	if ( $objItem->{DriveType} ne "3" )		# Data from non-local drives is variable.
	{
		$fix_detail_label = "var:Volume Management:file systems:$objItem->{Name}";
	}
	$fix_detail_label =~ s/:$//;		# erase colon in name
	$var_detail_label =~ s/:$//;		# erase colon in name

	print "$fix_detail_label"	. "::ProviderName	: $objItem->{ProviderName}\n";
	print "$fix_detail_label"	. "::DriveType	: $drive_type_list{ $objItem->{DriveType} }\n";
	print "$fix_detail_label"	. "::VolumeName	: $objItem->{VolumeName}\n";
	print "$fix_detail_label"	. "::SerialNumber	: $objItem->{VolumeSerialNumber}\n";
	print "$fix_detail_label"	. "::Description	: $objItem->{Description}\n";			# Type
	print "$fix_detail_label"	. "::FileSystem	: $objItem->{FileSystem}\n";
	print "$fix_detail_label"	. "::Size		: $size\n";
	print "$var_detail_label"	. "::Free Space	: $free\n";
	print "$var_detail_label"	. "::Used		: $perc_used\n";
	print "$var_detail_label"	. "::Avail		: $perc_free\n";
}

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

exit 0;
