# Based on code generated by ScriptoMatic from MicroSoft.

# Perl script to collect software data.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.17 $

my $PROGNAME = "$0";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME \n";
}

my $tmp_1_file = "scc_0600_1";
my $tmp_2_file = "scc_0600_2";

use strict;

use Win32::OLE('in');

use constant wbemFlagReturnImmediately => 0x10;
use constant wbemFlagForwardOnly => 0x20;

my $computer = $ENV{ SCC_WMI_COMPUTER };

# Optional: Change the delimiter to '/' to eliminate all of the extra backslashes 
use Win32::TieRegistry (Delimiter => "/"); #%apps will have a list of applications 
my %apps; 
my %version_l;
my %install_date_l;
my %install_loc_l;
my %publisher_l;
my %no_modify_l;
my %no_remove_l;
my %no_repair_l;
my %parent_key_l;
my %windows_inst_l;

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "	start retrieving software from registry\n";
}
# $softKey points to the Uninstall key that has the Add/Remove Programs information
my $softKey = $Registry->{"//$computer/HKEY_LOCAL_MACHINE/Software/Microsoft/Windows/CurrentVersion/Uninstall"};
# Sort and iterate through the keys under Uninstall 
foreach(keys %{$softKey}) 
{
	#Append the key and DisplayName to the Uninstall key
	#Set the result as a key in the %apps hash
	#This also removes any duplicate entries
	my $name = $softKey->{"$_/DisplayName"};
	# Avoid any colon in the name:
	$name =~ s/:/_/g;
	if ( $name ne "" )
	{
		$apps{$name} = 1;					# Indicate presence
		$version_l{$name} = $softKey->{"$_/DisplayVersion"};
		$install_date_l{$name} = $softKey->{"$_/InstallDate"};
		$install_loc_l{$name} = $softKey->{"$_/InstallLocation"};
		$no_modify_l{$name} = $softKey->{"$_/NoModify"};
		$no_remove_l{$name} = $softKey->{"$_/NoRemove"};
		$no_repair_l{$name} = $softKey->{"$_/NoRepair"};
		$parent_key_l{$name} = $softKey->{"$_/ParentKeyName"};
		$publisher_l{$name} = $softKey->{"$_/Publisher"};
		$windows_inst_l{$name} = $softKey->{"$_/WindowsInstaller"};
	}
}

print "hlp:software:product::Data from registry: HKLM/Software/Microsoft/Windows/CurrentVersion/Uninstall\n";
# Print a sorted list of the keys we got; ignore case.
foreach my $product ( sort {lc($a) cmp lc($b)} keys %apps )
{
	# Remove CR/LF from label.
	my $f_label = "fix:software:product:$product";
	$f_label =~ s/\012/_/g;
	$f_label =~ s/\015/_/g;
	my $v_label = "var:software:product:$product";
	$v_label =~ s/\012/_/g;
	$v_label =~ s/\015/_/g;

	print "$f_label" .     "::Version			: $version_l{$product} \n";
	print "$f_label" .     "::Publisher		: $publisher_l{$product} \n";
	print "$f_label" .     "::ParentKey		: $parent_key_l{$product} \n";
	print "$v_label" .     "::Install Date		: $install_date_l{$product} \n";
	print "$f_label" .     "::Install Location	: $install_loc_l{$product} \n";

	# Next items are empty for most products; test whether to print them.
	if ( $no_modify_l{$product} )
	{
		print "$f_label" .     "::NoModify		: $no_modify_l{$product} \n";
	}
	if ( $no_remove_l{$product} )
	{
		print "$f_label" .     "::NoRemove		: $no_remove_l{$product} \n";
	}
	if ( $no_repair_l{$product} )
	{
		print "$f_label" .     "::NoRepair		: $no_repair_l{$product} \n";
	}
	if ( $windows_inst_l{$product} )
	{
		print "$f_label" .     "::Windows Installer	: $windows_inst_l{$product} \n";
	}
}

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

unlink $tmp_1_file;

exit 0 ;
